/***************************************************************************
 *   Copyright (c) 2023 Ondsel, Inc.                                       *
 *                                                                         *
 *   This file is part of OndselSolver.                                    *
 *                                                                         *
 *   See LICENSE file for details about copyright.                         *
 ***************************************************************************/
 
#pragma once

#include "ExpressionX.h"
#include "System.h"
#include "Units.h"

namespace MbD {
    class ASMTTime : public ExpressionX
    {
        //
    public:
        static std::shared_ptr<ASMTTime> With();
        ASMTTime() = default;
        ASMTTime(Symsptr arg);
        void deleteMbD();
        void createMbD(std::shared_ptr<System> mbdSys, std::shared_ptr<Units> mbdUnits) override;
        Symsptr expandUntil(Symsptr sptr, std::shared_ptr<std::unordered_set<Symsptr>> set) override;
        Symsptr simplifyUntil(Symsptr sptr, std::shared_ptr<std::unordered_set<Symsptr>> set) override;
        Symsptr copyWith(Symsptr arg) override;
        bool isVariable() override;
        void setValue(double val) override;

    };
}

